
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: Minimap, MinimapCluster, next, UIParent

local MouseIsOver = MouseIsOver
local SixUI_Minimap = SixUI_Minimap
local SixUI_MinimapButtons = SixUI_MinimapButtons
local UIFrameFadeIn = UIFrameFadeIn
local UIFrameFadeOut = UIFrameFadeOut
local ZoneText = SixUI_MinimapButtons.ZoneText

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Minimap Frame")

local MINIMAP_SCALE = SixUI_Minimap:GetHeight() / Minimap:GetHeight()

-------------------------------------------------------------------------------
-- Helpers
-------------------------------------------------------------------------------

local function Minimap_OnUpdate(self)
	if not MouseIsOver(SixUI_Minimap) then
		SixUI_Minimap:SetScript("OnUpdate", nil)
		UIFrameFadeOut(SixUI_MinimapButtons, 0.25, SixUI_MinimapButtons:GetAlpha(), 0)
		UIFrameFadeIn(ZoneText, 0.25, ZoneText:GetAlpha(), 1)
	end
end

local function Minimap_OnEnter(self)
	UIFrameFadeIn(SixUI_MinimapButtons, 0.25, SixUI_MinimapButtons:GetAlpha(), 1)
	UIFrameFadeOut(ZoneText, 0.25, ZoneText:GetAlpha(), 0)
	SixUI_Minimap:SetScript("OnUpdate", Minimap_OnUpdate)
end

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnInitialize()
	self:RegisterEvent("UI_SCALE_CHANGED", "ConfigureMinimapBorder")
	self:RegisterFrames()
	self:ConfigureMinimapBorder()
	self:ConfigureMinimapButtons()
	self:ConfigureMinimapCluster()

	if not self:IsHooked(MinimapCluster, "SetHeaderUnderneath") then
		self:SecureHook(MinimapCluster, "SetHeaderUnderneath", "MinimapCluster_SetHeaderUnderneath")
	end
end

function M:ConfigureMinimapBorder()
	B:CreateBorder(SixUI_Minimap, 2)
	SixUI_Minimap:SetScale(UIParent:GetScale())
end

function M:ConfigureMinimapButtons()
	if not self:IsHooked(Minimap, "OnEnter") then
		self:HookScript(Minimap, "OnEnter", Minimap_OnEnter)
	end
end

function M:ConfigureMinimapCluster()
	MinimapCluster.minimumWidth, MinimapCluster.minimumHeight = MinimapCluster.MinimapContainer:GetSize()
	MinimapCluster.maximumWidth, MinimapCluster.maximumHeight = MinimapCluster.MinimapContainer:GetSize()
	MinimapCluster:SetClampedToScreen(false)
	MinimapCluster:SetScale(MINIMAP_SCALE)
	Minimap:SetMaskTexture([[Interface\ChatFrame\ChatFrameBackground]])

	for _, frame in next, {MinimapCluster:GetChildren()} do
		if frame ~= MinimapCluster.MinimapContainer then
			frame:Hide()
		end
	end
end

function M:RegisterFrames()
	A:RegisterFrame(Minimap.ZoomIn):Hide()
	A:RegisterFrame(Minimap.ZoomOut):Hide()
	A:RegisterFrame("MinimapBackdrop"):Hide()
	A:RegisterFrame("MiniMapMailIcon"):Hide()
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function M:MinimapCluster_SetHeaderUnderneath(frame, headerUnderneath)
	if not headerUnderneath then
		frame:SetHeaderUnderneath(true)
	end
end
